import { defineStore } from 'pinia'
import audio from '~creek-website-toolkit/lib/audio.js'

import pinia from '~creek-website-toolkit/stores/pinia-init.js'
import { useStudioStore } from '~creek-website-toolkit/stores/StudioStore.js'


const X = console.log

let audioElement = audio.createAudioContainer()

export const useGlobalPlayerStore = defineStore('globalPlayer', {
  state: () => ({
    sources: {
      broadcast: null,
      stream: null,
    },
    meta: {
      show: null,
    },
    sourceType: null,
    hasStream: null,
    hasBroadcast: null,
    error: null,
    isLoading: false,
    isPlaying: false,
    seek: null,
    duration: 1,
    position: 0,
    URL: null,
    audioElement: audioElement,
    areAudioEventListenersCreated: false,
    updatePositionInterval: null,
  }),

  actions: {
    activateBroadcast(payload) {
      X('activateBroadcast')

      if (this.sources.broadcast && this.sources.broadcast.id == payload.id) {
        X('same broadcast')
        return false
      }

      this.sourceType = 'audio'
      this.sources.stream = null
      this.sources.broadcast = payload
      this.hasStream = false
      this.hasBroadcast = true
      this.URL = payload.audio.url
      this.position = 0
    },

    deactivateBroadcast() {
      this.sources.broadcast = null
      this.hasBroadcast = false
      this.URL = null
      this.position = 0
    },

    activateStream(payload) {
      if (!payload) {
        return
      }

      this.sourceType = 'audio'
      this.sources.stream = payload
      this.sources.broadcast = null
      this.hasStream = true
      this.hasBroadcast = false
      this.URL = payload.url + '?nocache=' + Math.random()
      this.position = null
    },

    onPlay() {
      this.isPlaying = true
    },

    onPause() {
      this.isPlaying = false
    },

    onLoadStart() {
      this.isLoading = true
    },

    onAbort() {
      this.isLoading = false
    },

    onLoadSuccess(payload) {
      this.isLoading = false

      if (payload) {
        this.duration = payload.duration
      }

      this.error = null
    },

    onLoadFailure(payload) {
      this.error = payload
    },

    onSeek(payload) {
      this.seek = payload
      X(payload)
      this.isPlaying = true
    },

    onSlidingStart() {
      this.isPlaying = false
    },

    onPlaybackComplete() {
      this.isPlaying = false
    },

    clearError() {
      this.error = null
    },

    setAudioEventListenersCreated() {
      this.areAudioEventListenersCreated = true
    },

    updatePosition() {
      this.position = audioElement.currentTime
    },

    setUpdatePositionInterval(callback) {
      if (this.updatePositionInterval) {
        clearInterval(this.updatePositionInterval)
      }
      this.updatePositionInterval = setInterval(() => {
        if (callback) {
          callback()
        }
      }, 1000)
    },

    clearUpdatePositionInterval() {
      if (this.updatePositionInterval) {
        clearInterval(this.updatePositionInterval)
      }
    },

    async play() {
      if (!audioElement || !audioElement.play) {
        return false
      }

      this.onLoadStart()
      this.updateNotificationMetadata()
      audioElement.play()
    },

    async pause() {
      if (!audioElement || !audioElement.pause) {
        X('no pause')
        return false
      }

      audioElement.pause()
      this.stopUpdatePositionInterval()
    },

    async stop() {
      if (!audioElement || !audioElement.stop) {
        X('no stop')
        return false
      }

      audioElement.stop()
      this.stopUpdatePositionInterval()
    },

    async playBroadcast(broadcast) {
      this.activateBroadcast(broadcast)

      try {
        await this.loadActivatedSource()
      } catch (error) {
        X(error)
      }

      X('wib wob')
      await this.play()
    },

    async playStream(stream = null) {

      const studioStore = useStudioStore(pinia)

      if (!stream) {
        stream = studioStore.streamPrimary
      }

      X(stream)
      this.activateStream(stream)

      try {
        await this.loadActivatedSource()
      } catch (error) {
        X(error)
      }

      X('wib wob')
      await this.play()
    },

    async backToMainStream() {
      this.deactivateBroadcast()
      await this.playStream(null)
    },

    async loadActivatedSource() {
      this.startUpdatePositionInterval()

      if (!audioElement) {
        X('audioElement not loaded')
        return false
      }

      X('loading audio url:')
      X(this.URL)

      X(audioElement)
      X(audioElement.src)
      X(audioElement.currentSrc)

      let sourceURL = this.URL

      if (audioElement.currentSrc == sourceURL) {
        X('same URL')
        return true
      }

      if (this.hasStream) {
        sourceURL = `${sourceURL}?no-cache=${Math.random()}`
        X('adding no-cache query string:')
        X(sourceURL)
      }

      let shouldIgnoreFirefoxPageChangeError = false
      window.addEventListener('beforeunload', (event) => {
        shouldIgnoreFirefoxPageChangeError = true
      })

      audioElement.src = sourceURL

      if (!this.areAudioEventListenersCreated) {
        this.setAudioEventListenersCreated()

        if (!audioElement) {
          X('no audioElement for event listeners')
          return
        }

        audioElement.addEventListener('loadeddata', event => {
          X('audio event: loadeddata')
        })

        audioElement.addEventListener('loadedmetadata', event => {
          X('audio event: loadeddata')
        })

        audioElement.addEventListener('canplay', event => {
          X('audio event: canplay')
          this.onLoadSuccess()
        })

        audioElement.addEventListener('canplaythrough', event => {
          X('audio event: canplaythrough')
        })

        audioElement.addEventListener('durationchange', event => {
        })

        audioElement.addEventListener('play', event => {
          X('audio event: play')
          this.onLoadSuccess()
          this.onPlay()
        })

        audioElement.addEventListener('pause', event => {
          X('audio event: pause')
          this.onPause()
        })

        audioElement.addEventListener('progress', event => {
        })

        audioElement.addEventListener('seeked', event => {
          X('audio event: seeked')
        })

        audioElement.addEventListener('seeking', event => {
          X('audio event: seeking')
        })

        audioElement.addEventListener('stalled', event => {
          X('audio event: stalled')
        })

        audioElement.addEventListener('error', event => {
          X('audio event: error')
          if (shouldIgnoreFirefoxPageChangeError) {
            return false
          }
          this.onLoadFailure(event)
          X(event)
          X(event.message)
        })

        audioElement.addEventListener('abort', event => {
          X('audio event: abort')
          this.onAbort()
        })

        audioElement.addEventListener('suspend', event => {
          X('audio event: suspend')
        })

        audioElement.addEventListener('timeupdate', event => {
        })

        audioElement.addEventListener('volumechange', event => {
          X('audio event: volumechange')
        })

        audioElement.addEventListener('waiting', event => {
          X('audio event: waiting')
          this.onLoadStart()
        })

        audioElement.addEventListener('ended', event => {
          X('audio event: ended')
          this.onPlaybackComplete()
        })

        audioElement.addEventListener('playing', event => {
          this.onPlay()
        })

        audioElement.addEventListener('complete', event => {
          X('audio event: complete')
        })
      }
    },

    startUpdatePositionInterval() {
      if (this.updatePositionInterval) {
        this.clearUpdatePositionInterval()
      }
      this.setUpdatePositionInterval(() => this.updatePosition())
    },

    stopUpdatePositionInterval() {
      if (this.updatePositionInterval) {
        this.clearUpdatePositionInterval()
      }
    },

    seek(seekPayload) {
      X('seek')
      X(seekPayload)

      if (this.sources.stream) {
        return false
      }

      if (!audioElement) {
        X('no audioElement to seek')
        return false
      }

      audioElement.currentTime = seekPayload
    },

    updateNotificationMetadata() {

      const studioStore = useStudioStore()

      if (!navigator.mediaSession) {
        return
      }

      X('Updating media controls notification:')

      let title = null

      if (studioStore.current.song) {
        title = studioStore.current.song.title
      } else if (this.sources.broadcast) {
        title = this.sources.broadcast.title || this.sources.broadcast.show.title
      }

      let artist = null

      if (studioStore.current.song) {
        artist = studioStore.current.song.artist
      } else if (this.sources.broadcast) {
        artist = this.sources.broadcast.title || this.sources.broadcast.show.title
      }

      let album = null

      if (this.sources.broadcast) {
        album = this.sources.broadcast.title || this.sources.broadcast.show.title
      } else if (this.sources.stream) {
        album = studioStore.studio.title
      }
    },
  },
})