import { defineStore } from 'pinia'
import axios from 'axios'

import pinia from '~creek-website-toolkit/stores/pinia-init.js'
import { useGlobalPlayerStore } from '~creek-website-toolkit/stores/GlobalPlayerStore.js'
import { get, capitalize } from '~creek-website-toolkit/lib/utils'


const X = console.log

const handleError = (error) => {
  errorMessage = error.message || error
  X('Error: ' + errorMessage)
}

const handleAxiosResponseError = (axiosError) => {
  if (get(axiosError, 'response.data.error')) {
    // console.log('Creek: Error:')
    // console.log(axiosError.response.data.error)
    Creek.response.error = axiosError.response.data.error
  }

  if (get(axiosError, 'response.data.errors')) {
    // console.log('Creek: Errors:')
    // console.log(axiosError.response.data.errors)
    Creek.response.errors = axiosError.response.data.errors
  }

  return null
}

const responseHandler = async (axiosRequest) => {
  let response = null

  try {
    response = await axiosRequest
  } catch (responseError) {
    return handleAxiosResponseError(responseError)
  }

  if (response.data && response.data.error) {
    handleError(response.data.error)
    return null
  }

  if (response.data && response.data.data) {
    return response.data.data
  } else if (response.data) {
    return response.data
  }
}

export const useStudioStore = defineStore('studio', {
  state: () => ({
    studio: null,
    domain: null,
    prefix: null,
    studioName: null,
    studioId: null,
    version: null,
    current: {},
    times: [],
    shows: [],
    show: null,
    broadcast: null,
    broadcasts: [],
    recentBroadcasts: [],
    streams: [],
    streamPrimary: null,
    UI: {
      isChatToolbarOpen: false,
      isVideoToolbarOpen: false,
      broadcastToolbarHeight: 0,
      isWebAppInstructionsBoxVisible: false,
    },
    showsStringToFind: '',
    scheduleWeekdayNumber: null,
    showsPath: '/shows',
    scheduleViewSelected: 'schedule',
    isDev: false,
  }),

  getters: {

    getURLFromPath: (state) => (path) => {
      if (state.isDev) {
        state.prefix = 'http://dev-studio.creek.org'
      } else {
        state.prefix = 'https://studio.creek.org'
      }
      path = path.replace('__VERSION__', 'v' + state.version || '1.0.1')
      const glue = path.includes('?') ? '&' : '?'
      const studioURL = state.prefix + path + glue + 'studioId=' + state.studioId
      return studioURL
    },

    getEmbedURLFromPath: (state) => (path) => {
      if (state.isDev) {
        state.prefix = 'http://dev-embed.creek.org'
      } else {
        state.prefix = 'https://embed.creek.org'
      }
      path = path.replace('__VERSION__', 'v' + state.version || '1.0.1')
      const studioURL = state.prefix + path + '?studioId=' + state.studioId
      return studioURL
    },
    
  },

  actions: {
    setDomain(payload) {
      this.domain = payload
    },

    setStudioId(studioId) {
      this.studioId = studioId
    },

    setDevMode(payload) {
      this.isDev = payload
    },

    setVersion(payload) {
      this.version = payload
    },

    setStudio(payload) {
      this.studio = payload
    },

    setCurrent(payload) {
      this.current = payload
    },

    setStreams(payload) {
      this.streams = payload

      const streams = payload
      let streamPrimary = null

      if (!streams) {
        return null
      }

      for (let stream of streams) {
        if (stream.purpose == 'main') {
          this.streamPrimary = stream
        }
      }

      if (!this.streamPrimary) {
        this.streamPrimary = streams[0]
      }
    },

    setRecentBroadcasts(payload) {
      this.recentBroadcasts = payload
    },

    toggleChatToolbar() {
      this.UI.isChatToolbarOpen = !this.UI.isChatToolbarOpen
    },

    openChatToolbar() {
      this.UI.isChatToolbarOpen = true
    },

    closeChatToolbar() {
      this.UI.isChatToolbarOpen = false
    },

    toggleVideoToolbar() {
      this.UI.isVideoToolbarOpen = !this.UI.isVideoToolbarOpen
    },

    toggleWebAppInstructionsBox() {
      this.UI.isWebAppInstructionsBoxVisible = !this.UI.isWebAppInstructionsBoxVisible
    },

    setShowsStringToFind(payload) {
      this.showsStringToFind = payload
    },

    setScheduleWeekdayNumber(payload) {
      X(payload)
      this.scheduleWeekdayNumber = payload
    },

    setShowsPath(payload) {
      // X(payload)
      this.showsPath = payload
    },

    setAuthorsPath(payload) {
      // X(payload)
      this.authorsPath = payload
    },

    selectScheduleView(payload) {
      // X(payload)
      this.scheduleViewSelected = payload
    },

    async init({ studioNameOrDomain = null, studioId = null }) {
      // X('creek/studio/init')
      // X(studioNameOrDomain)
      // X(studioId)

      let domain = null

      if (studioNameOrDomain && studioNameOrDomain.includes('.')) {
        domain = studioNameOrDomain
        this.studioName = studioNameOrDomain.split('.')[0]
      } else if (studioNameOrDomain) {
        domain = studioNameOrDomain + '.studio.creek.org'
        this.studioName = studioNameOrDomain
      } else {
        domain = 'studio.creek.org'
        this.studioId = studioId
        this.studioName = null
      }

      this.setStudioId(studioId)
      this.setDomain(domain)

      await this.getStudio()
      await this.getCurrent()
      await this.getStreams({ shouldActivate: true })
    },

    async getResource(
      {
        path = null,
        id = null,
        domain = null,
        shouldCommit = true,
        commitFunctionName = null,
      }
    ) {
      if (!this.domain) {
        throw Error('Creek: No domain set.')
      }

      if (!path) {
        throw Error('Creek: No resource path specified.')
      }

      let name = path.split('?')[0].split('/')[0]

      if (id) {
        path += '/' + id
      }

      let domainForRequest = this.domain

      if (domain) {
        domainForRequest = domain
      }

      let URLPrefix = `https://${domainForRequest}/api/`

      let axiosInstance = axios.create({
        baseURL: URLPrefix,
        timeout: 10000,
      })

      const studioId = this.studioId
      path = path + (path.includes('?') ? '&' : '?') + 'studioId=' + studioId

      // console.log('Creek: Getting resource:')
      // console.log(URLPrefix + path)

      const result = await responseHandler(axiosInstance.get(path))

      if (shouldCommit) {
        if (commitFunctionName) {
          this[commitFunctionName](result)
        } else {
          this[`set${capitalize(name)}`](result)
        }
      }

      return result
    },

    async getStudio() {
      this.setStudio(await this.getResource({ path: 'studio' }))
    },

    async getCurrent() {
      this.setCurrent(await this.getResource({ path: 'current?x=1' }))
    },

    async getStreams({ shouldActivate = false }) {
      const streams = await this.getResource({ path: 'streams' })
      this.setStreams(streams)
      if (shouldActivate) {
        // X('getStreams.shouldActivate')
        const globalPlayerStore = useGlobalPlayerStore(pinia)
        globalPlayerStore.activateStream(this.streamPrimary)
      }
    },

    async getTimes() {
      this.times = await this.getResource({ path: 'times' })
    },

    async getRecentBroadcasts() {
      this.setRecentBroadcasts(
        await this.getResource({
          path: 'broadcasts',
          resourcePath: 'broadcasts/not-expired',
        })
      )
    },

  },
})
